% MATLAB script by Pavlo Bazilinskyy and Joost de Winter
% questions/comments to <pavlo.bazilinskyy@tue.nl>

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Process appen and heroku data for the experiment. The function needs
% to be somewhat customised for each individual study. The working
% principle is however translatable across studies. Output should be
% customised to cater to all questions and types of data collected in the
% particular study.

%%% Naming convention:
% PP: participant
% appen: platform used to recruit participants and ask survey questions
%        (https://appen.com)
% heroku: platform used to run the experiment and store data
%         (https://www.heroku.com)
% mongo: data is stored in the MongoDB database (https://mlab.com). Note:
%        free account there allows storing 500MB of data. For a new study,
%        we monitor the size of the DB and save data in pieces of <500MB.
%
%%% Input:
% experiment_id: ID of experiment (multiple experiments can be fetched for
%                the same study to conduct cross-experiment analysis)
% save_mat_file: bool flag for saving mat file. A list of variables to save
%                needs to be adjusted based on output of the function
% name_mat_file: name of mat file with saved data
% load_mat_file: bool flag for loading mat file. If true, no analysis will
%                be done and the file with data will be loaded instead
% appen_file: csv file with appen data
% appen_indices: indices in appen data
% heroku_files: list of files with heroku data
% prefix_stimuli: prefix of saved location of stimuli
%                 (e.g., image/video/audio)
% num_stimuli: number of stimuli
% num_stimuli_block: number of stimuli per saved block
%
%%% Output:
% X: demographics and other numeric data for participants
% Country: countries of participants
% RT: reaction times/times of keypresses (legacy name 'reaction time')
% RP: data from slider/choice question (legacy name 'reaction press')
% StimuliIDs: IDs of stimuli as shown to participants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X, ...
    Country, ...
    RT, ...
    Sliders, ...
    StimulusIDs] = process_experiment(participant_filtering_type, ...
    save_mat_file, ...
    name_mat_file, ...
    appen_file, ...
    appen_indices, ...
    heroku_files, ...
    num_stimuli, ...
    num_stimuli_block)

%%% Constants
MISTAKES_THRESHOLD = 1;  % number of allowed mistakes with test phrases
%%% Process data instead of loading a previously saved mat file
if save_mat_file
    %%% Import data
    disp('Start importing data');
    % process multiple files with heroku data
    [~,~,raw_heroku] = xlsread(heroku_files);

    % import Excel file with appen data (survey page)
    raw_appen = readtable(appen_file, 'ReadVariableNames', false);
    raw_appen = table2cell(raw_appen);  % convert to cell array for ease of checking
    disp(['Imported data from file with appen data ' appen_file]);
    disp('End of importing data');
    %% Process appen data
    disp('Start of processing appen data');
    % allocate matrix to store numbers answers to survey questions, worker
    % ID and additional data/flags
    X=NaN(size(raw_appen,1),25);
    X(:,23)=0;  % set counter of mistakes in test phrases to 0
    disp(['Number of respondents in appen data = ' ...
        num2str(size(raw_appen, 1))])
    % instructions understood
    found_values=raw_appen(:,appen_indices(1));
    X(:,1)=strcmp(found_values,'no')+2*strcmp(found_values,'yes');
    % gender
    found_values=raw_appen(:,appen_indices(2));
    X(:,2)=1*strcmp(found_values,'female') ...
        + 2*strcmp(found_values,'male') ...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');
    % age
    found_values=raw_appen(:,appen_indices(3));
    for i=1:size(found_values,1)
        try if strcmp(found_values(i),'?')
                X(i,3)=NaN;
        else
            X(i,3)=cell2mat(found_values(i));
        end
        catch
            X(i,3)=NaN;
        end
    end
    X(X(:,3)>110,3)=NaN; % people who report age greater than 110 years
    % age of obtaining driver's license
    found_values=raw_appen(:,appen_indices(4));
    for i=1:size(found_values,1)
        try
            if strcmp(found_values(i),'?')
                X(i,4)=NaN;
            else
                X(i,4)=cell2mat(found_values(i));
            end
        catch
            X(i,4)=NaN;
        end
    end
    X(X(:,4)>110,4)=NaN; % people who report licence more than 110 years
    % primary mode of transportation
    found_values=raw_appen(:,appen_indices(5));
    X(:,5)= 1*strcmp(found_values,'private_vehicle') ...
        + 2*strcmp(found_values,'public_transportation') ...
        + 3*strcmp(found_values,'motorcycle') ...
        + 4*strcmp(found_values,'walkingcycling') ...
        + 5*strcmp(found_values,'other') ...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');
    % how many times in past 12 months did you drive a vehicle
    found_values=raw_appen(:,appen_indices(6));
    X(:,6)=1*strcmp(found_values,'never') ...
        + 2*strcmp(found_values,'less_than_once_a_month') ...
        + 3*strcmp(found_values,'once_a_month_to_once_a_week') ...
        + 4*strcmp(found_values,'1_to_3_days_a_week') ...
        + 5*strcmp(found_values,'4_to_6_days_a_week') ...
        + 6*strcmp(found_values,'every_day') ...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');
    % mileage
    found_values=raw_appen(:,appen_indices(7));
    X(:,7)=1*strcmp(found_values,'0_km__mi') ...
        + 2*strcmp(found_values,'1__1000_km_1__621_mi') ...
        + 3*strcmp(found_values,'1001__5000_km_622__3107_mi') ...
        + 4*strcmp(found_values,'5001__15000_km_3108__9321_mi') ...
        + 5*strcmp(found_values,'15001__20000_km_9322__12427_mi') ...
        + 6*strcmp(found_values,'20001__25000_km_12428__15534_mi') ...
        + 7*strcmp(found_values,'25001__35000_km_15535__21748_mi') ...
        + 8*strcmp(found_values,'35001__50000_km_21749__31069_mi') ...
        + 9*strcmp(found_values,'50001__100000_km_31070__62137_mi') ...
        + 10*strcmp(found_values,'more_than_100000_km_more_than_62137_mi') ...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');
    % number of accidents
    found_values=string(raw_appen(:,appen_indices(8)));
    X(:,8)=1*strcmp(found_values,'0') ...
        + 2*strcmp(found_values,'1') ...
        + 3*strcmp(found_values,'2') ...
        + 4*strcmp(found_values,'3') ...
        + 5*strcmp(found_values,'4') ...
        + 6*strcmp(found_values,'5') ...
        + 9*strcmp(found_values,'more_than_5') ...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');
    % country
    found_values=raw_appen(:,appen_indices(9));
    Country=cell(size(X,1),1);
    for i=1:size(found_values,1)
        try
            % try to extract country code
            Country(i)=unique(found_values(i));
        catch error
            Country(i)={'NaN'};
        end
    end
    % driver behaviour questionnaire (DBQ)
    found_values=raw_appen(:,appen_indices(10:16));
    X(:,9:15)=1*strcmp(found_values,'0_times_per_month') ...
        + 2*strcmp(found_values,'1_to_3_times_per_month') ...
        + 3*strcmp(found_values,'4_to_6_times_per_month') ...
        + 4*strcmp(found_values,'7_to_9_times_per_month') ...
        + 5*strcmp(found_values,'10_or_more_times_per_month')...
        - 1*strcmp(found_values,'i_prefer_not_to_respond');

    % headphones
    found_values=raw_appen(:,appen_indices(21));
    X(:,20)=strcmp(found_values,'no')+2*strcmp(found_values,'yes')-1*strcmp(found_values,'i_prefer_not_to_respond');

    % problems with hearing
    found_values=raw_appen(:,appen_indices(22));
    X(:,21)=strcmp(found_values,'no')+2*strcmp(found_values,'yes')-1*strcmp(found_values,'i_prefer_not_to_respond');

    % set -1 responses (prefer not to respond, missing data) values to NaN

    X(X<0)=NaN;
    %%% Survey time
    for i=1:size(raw_appen, 1)
        starttime=datenum(raw_appen{i,appen_indices(17)});
        endtime=datenum(raw_appen{i,appen_indices(18)});
        X(i,16)=starttime; % save start time
        X(i,17)=endtime;  % save end time
        X(i,18)=round(2400*36*(endtime - starttime));  % survey time
    end


    disp(['Survey time mean (minutes) before filtering = ' num2str(nanmean(X(:,18)/60))]);
    disp(['Survey time median (minutes) before filtering = ' num2str(nanmedian(X(:,18)/60))]);
    disp(['Survey time SD (minutes) before filtering = ' num2str(nanstd(X(:,18)/60))]);
    disp(['First survey start date before filtering = ' datestr(min(X(:,16)))]);
    disp(['Last survey end date before filtering = ' datestr(max(X(:,17)))]);
    %%% Worker id
    found_ids=raw_appen(:,appen_indices(19));
    for i=1:size(found_ids,1)
        try if strcmp(found_ids(i),'?')
                X(i,19)=NaN;
        else
            X(i,19)= cell2mat(found_ids(i));
        end
        catch
            X(i,19)=NaN;
        end
    end

    disp('End of processing appen data');

    %%% Process keypress data
    % assumed max possible number of values for 1 stimulus
    num_in_row = 2000;
    % reaction time values for participants
    RT = NaN(size(raw_appen,1), num_stimuli, num_in_row);
    % keypress values for participants (slider questions)
    Sliders = NaN(size(raw_appen,1), num_stimuli, 3);
    % IDs of stimuli shown
    StimulusIDs = NaN(size(raw_appen,1), num_stimuli);
    % flags for blocks that were found. displaying the state of this matrix
    % at the end of the function can be used to debug if all data was parsed
    % successfully
    found_values = zeros(size(raw_appen,1), num_stimuli/num_stimuli_block);
    % count people that added the same worker_code more than once
    counter_cheaters = 0;
    cheater_worker_codes=[];  % store worker_codes of cheaters
    % map of correct answers to the test phrase
    test_sounds_answers = ["", ""; ...  % 1 (this is the music)
        "ora", "ora"; ...  % 2
        "lem", "yel"; ...  % 3
        "che", "red"; ...  % 4
        "app", "gre"; ...  % 5
        "ber", "bla"; ...  % 6
        "gra", "blu"];     % 7

    % remove non chars (a-z) and (A-Z)
    for i=1:size(test_sounds_answers,1)
        for j=1:size(test_sounds_answers,2)
            s = test_sounds_answers{i,j};
            s(isletter(s)==0)=[];
            test_sounds_answers(i,j) = s;
        end
    end
    disp('Start of processing heroku data');
    for row_heroku=1:size(raw_heroku,1) % loop over rows
        Weird=0; % flag to mark whether to
        if row_heroku/100 == round(row_heroku/100)
            disp(['Row: ' num2str(row_heroku) ' out of ' num2str(size(raw_heroku,1))])
        end
        % store data from the current row to add to the correct place in X
        % RT values in the row. Allocate num_in_row for multiple RT values
        RT_row = NaN(num_stimuli_block,num_in_row);   % keypress values in row
        Sliders_row = NaN(num_stimuli_block,3);  % slider values in the row
        stimuli_id_row = NaN(num_stimuli_block,1);  % stimulus ID values in row
        extracted_row = raw_heroku(row_heroku,:);  % row of data in heroku
        counter_rt = 0;  % counter for reaction time
        counter_stimuli_id = 0;  % counter for stimuli IDs
        test_sound_detected = false;  % flag for detected test sound
        test_sound_detected_id = -1;  % id of the last detected test sound
        counter_mistakes_test_sounds = 0;  % counter number of mistakes made with test phrases
        rts_detected = false;  % flag for starting to record RTs of stimulus
        heroku_code = '';  % worker_code in heroku data
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Go over rows of heroku data. Process possible values in the cell
        % one by one and parse values in different ways. We assume that 1
        % cell can contain only 1 type of data. If the study is setup to
        % allow parsing the same cell in multiple ways, 'continue'
        % statements should be removed.
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for cell_counter=1:length(extracted_row)
            % work with 1 particular cell from the row of data
            cell_row=extracted_row{cell_counter};
            % skip NaN cell
            if isnan(cell_row)
                break;  % skip this cell
            end
            if contains(cell_row, 'key:')
                continue;  % skip this cell
            end
            % check if keypress times for stimulus will follow
            if contains(cell_row, 'rts:[')
                rts_detected = true;  % flag to start recording RTs from stimulus
                counter_rt = 0;  % reset counter of RTs for stimulus
                % no continue, as an rt value can follow in the cell
            end
            % stimuli ID

            % keypress time
            if contains(cell_row, '"rt"') && rts_detected % keypress time found
                % Record time key press from row array into key presses
                counter_rt = counter_rt + 1;
                RT_row(counter_stimuli_id+1,counter_rt) = sscanf(cell_row(1+max(strfind(cell_row,':')):end),'%f'); % Store RT in RT_row
                continue
            end
            if contains(cell_row,'stimulus:"[\"')
                rts_detected = false;
                stimuli_id_row(counter_stimuli_id + 1) = sscanf(cell_row(strfind(cell_row,'_')+1:max(strfind(cell_row,'.wav'))-1),'%f');
                counter_stimuli_id = counter_stimuli_id + 1;
                continue
            end       % worker code
            if contains(cell_row, '{"worker_code":"') % worker_code found
                heroku_code = cell_row(strfind(cell_row,'"worker_code":"')+15:max(strfind(cell_row,'"')-1));
                continue
            end

            if contains(cell_row,'slider-0') % responses to slider question found
                if counter_stimuli_id==0
                    disp(['Row ' num2str(row_heroku) '; Weird row: No stimulus identified yet'])
                    Weird=1;
                    break
                end
                Sliders_row(counter_stimuli_id, 1) = sscanf(cell_row(strfind(cell_row,'":\"')+4:max(strfind(cell_row,'\'))-1),'%f')/10;
                continue
            end

            if contains(cell_row,'slider-1') % responses to slider question found
                if counter_stimuli_id==0
                    disp(['Row ' num2str(row_heroku) '; Weird row: No stimulus identified yet'])
                    Weird=1;
                    break
                end
                Sliders_row(counter_stimuli_id, 2) = sscanf(cell_row(strfind(cell_row,'":\"')+4:max(strfind(cell_row,'\'))-1),'%f')/10;
                continue % only 1 item can be picked up from the cell
            end

            if contains(cell_row,'slider-2') % responses to slider question found
                if counter_stimuli_id==0
                    disp(['Row ' num2str(row_heroku) '; Weird row: No stimulus identified yet'])
                    Weird=1;
                    break
                end
                Sliders_row(counter_stimuli_id, 3) = sscanf(cell_row(strfind(cell_row,'":\"')+4:max(strfind(cell_row,'\'))-1),'%f')/10;
                continue % only 1 item can be picked up from the cell
            end
            % test phrase detected
            if contains(cell_row, 'sound_test_')
                test_sound_detected = true;
                test_sound_detected_id = sscanf(cell_row(max(strfind(cell_row,'_'))+1:max(strfind(cell_row,'.wav'))-1),'%f');
                continue % only 1 item can be picked up from the cell
            end

            % input test phrase detected
            if contains(cell_row, 'input-codeblock') && test_sound_detected
                % answer given by the participant
                given_answer = cell_row(35:length(cell_row)-4);

                % set to lowercase
                given_answer = lower(given_answer);

                % check if at least one of the possible sentences
                % contains a given answer
                if ~(contains(given_answer,test_sounds_answers(test_sound_detected_id,1)) || contains(given_answer,test_sounds_answers(test_sound_detected_id,2)))
                    % increase counter of mistakes
                    counter_mistakes_test_sounds = counter_mistakes_test_sounds + 1;
                end
                % uncheck flag that test sound was detected and reset
                % id of the sound
                test_sound_detected = false;
                test_sound_detected_id = -1;
                continue;  % only 1 item can be picked up from the cell
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Add data to corresponding row in X
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % find occurrences of the same worker_code in appen data
        row_appen_matched=find(strcmp(heroku_code, ...
            raw_appen(:,appen_indices(20))));
        if (counter_stimuli_id < num_stimuli_block || counter_stimuli_id > num_stimuli_block)
            % unexpected number of data points found
            Weird=1;
            disp(['Row ' num2str(row_heroku) '; Weird row: Weird number of stimuli in block'])
        end
        if ~isempty(row_appen_matched) && Weird==0 % (if no match, then likely the participant started but did not complete the task)
            % multiple instances of worker_code found
            if length(row_appen_matched) > 1
                % found a cheater
                if ~any(strcmp(cheater_worker_codes, heroku_code))
                    % worker_code was not marked as cheater before
                    % add to the list of known cheaters
                    cheater_worker_codes = [cheater_worker_codes, ...
                        heroku_code];
                    % increase counter of cheaters
                    counter_cheaters = counter_cheaters + 1;
                end
            end

            % for storing data, use only 1st occurrence of worker_id in
            % appen data
            row_appen_matched=row_appen_matched(1);

            % 1st block
            if found_values(row_appen_matched,1) == 0
                block_id = 1;
                found_values(row_appen_matched, 1) = 1;
                % this row
                % flag that row was matched
                X(row_appen_matched, 24) = 1;
                % not 1st block (2...n block)
            else
                % find last processed block
                last_block = find(found_values(row_appen_matched,:), ...
                    1, ...  % if 1, it was already processed
                    'last');  % get only last occurrence of 1
                % new block is last block + 1
                block_id = last_block + 1;
                % check if block_id is within allowed range
                if (block_id > num_stimuli/num_stimuli_block)
                    continue;
                end
                % update cell in matrix of processed blocks
                found_values(row_appen_matched, block_id) = 1;
            end
            % mistakes for test phrases
            X(row_appen_matched, 23) = X(row_appen_matched, 23) + counter_mistakes_test_sounds;

            % (1:values_to_save) is used for cases where more than
            % allowed number of stimuli is stored in a row in heroku.
            % it is rare but possible

            values_to_save = counter_stimuli_id;

            % index of 1st value
            start = block_id * num_stimuli_block - (num_stimuli_block - 1);
            % index of last value
            finish = start + (values_to_save - 1);
            RT(row_appen_matched, start:finish, :) = RT_row(1:values_to_save, :);
            Sliders(row_appen_matched, start:finish, :) = Sliders_row(1:values_to_save, :);
            StimulusIDs(row_appen_matched, start:finish) = stimuli_id_row(1:values_to_save);
        end % end of ~isempty(row_appen_matched) && Weird==0
    end % end of loop over rows
    disp(['End of processing heroku data']);

    %%% Find participants who did not meet the criteria
    % respondents who did not read instructions
    invalid1 = find(X(:,1)==1);
    % respondents who indicated they are under 18 years old
    invalid2 = find(X(:,3)<18);
    % respondents who took less than 15 min to complete
    invalid3 = find(X(:,18)<60*15);
    % respondents with no response data / match
    invalid4 = find(X(:,24)~=1);
    % respondents with hearing problems
    invalid5= find(X(:,21)==2);

    % respondents with little or no data
    invalid6 = find(sum(found_values, 2) < (num_stimuli / num_stimuli_block) / 2);

    %%% Find rows with identical IP addresses
    % store participants with multiple IPs
    pp_multiple_ips=NaN(size(X(:,1)));
    ip_appen=NaN(size(raw_appen,1),1);  % IP addressed in appen data
    % reduce IP addresses of appen data to a single number
    for i=1:size(raw_appen,1)
        try
            ip_appen(i)=str2double(strrep(raw_appen(i,appen_indices(23)),'.',''));
        catch
            ip_appen(i)=cell2mat(raw_appen(i,appen_indices(23)));
        end
    end

    for i=1:size(X,1) % loop over all Appen rows
        found_ips=find(ip_appen==ip_appen(i));
        % if the IP address occurs only once
        if length(found_ips)==1
            % only IP found, so keep
            pp_multiple_ips(i)=1;
            % if the IP address occurs more than once
        elseif length(found_ips)>1
            % keep the first survey for that IP address
            pp_multiple_ips(found_ips(1))=1;
            % no not keep the other ones
            pp_multiple_ips(found_ips(2:end))=2;
        end
    end
    % respondents who completed the survey more than once, i.e., remove
    % the doublets
    invalid7=find(pp_multiple_ips>1);
    % respondents who made more mistakes with test phrases than allowed
    invalid8 = find(X(:,23)>MISTAKES_THRESHOLD);
    if participant_filtering_type==2 % more liberal filtering
        invalid = unique([invalid1; ...
            invalid2; ...
            invalid3; ...
            invalid4; ...
            invalid5;...
            invalid6]);
    else % more strict filtering
        invalid = unique([invalid1; ...
            invalid2; ...
            invalid3; ...
            invalid4; ...
            invalid5;...
            invalid6; ...
            invalid8]);
    end
    %%% Remove filtered out data
    % add together all invalid rows with data
    X(invalid,:)=[];  % remove invalid respondents
    Country(invalid)=[];  % remove invalid countries
    RT(invalid,:,:)=[];  % remove invalid data
    Sliders(invalid,:,:)=[];  % remove invalid data
    StimulusIDs(invalid,:)=[];  % remove invalid data
    % Save mat file

    save(name_mat_file, ...
        'X', ...
        'Country', ...
        'RT', ...
        'Sliders', ...
        'StimulusIDs', ...
        'invalid1', ...  % here and lower need to be saved for output
        'invalid2', ...
        'invalid3', ...
        'invalid4', ...
        'invalid5',...
        'invalid6', ...
        'invalid7', ...
        'invalid8', ...
        'invalid', ...
        'counter_cheaters', ...
        '-v7.3');  % to support saving file with size >2GB
    disp(['Saved mat file as ' name_mat_file]);
    %%% Load mat file instead of processing data
else
    load(name_mat_file);  % load mat file in the current directory
    disp(['Loaded mat file ' name_mat_file]);
end
%% Output with statistics and filtering information
disp(['Number of responses who could not be linked with Heroku database ' num2str(num_stimuli/2) ' stimuli = ' num2str(length(invalid6))]);
disp('With the following additional exclusions:')
disp(['Number of respondents who did not read instructions = ' num2str(length(setdiff(invalid1,invalid6)))])
disp(['Number of respondents under 18 = ' num2str(length(setdiff(invalid2,invalid6)))])
disp(['Number of respondents who took less than 900 s = ' num2str(length(setdiff(invalid3,invalid6)))])
disp(['Number of rows in keypress data not matched =  ' num2str(length(setdiff(invalid4,invalid6)))]);
disp(['Number of respondents with hearing problems =  ' num2str(length(setdiff(invalid5,invalid6)))]);
disp(['Number of responses coming from the same IP = ' num2str(length(setdiff(invalid7,invalid6)))]);
disp(['Number of responses who made mistakes with test phrases = ' num2str(length(setdiff(invalid8,invalid6)))]);
disp(['Number of respondents removed = ' num2str(length(setdiff(invalid,invalid6)))]);
disp(['Number of respondents removed in total = ' num2str(length(invalid))]);
disp(['Number of respondents included in the analysis = ' num2str(size(X,1))]);
disp(['Number of countries included in the analysis =  ' num2str(length(unique(Country)))]);
%% Gender, age
disp(['Gender, male respondents = ' num2str(sum(X(:,2)==2))])
disp(['Gender, female respondents = ' num2str(sum(X(:,2)==1))])
disp(['Gender, I prefer not to respond = ' num2str(sum(isnan(X(:,2))))])
disp(['Age, mean = ' num2str(mean(X(:,3),'omitnan'))])
disp(['Age, median = ' num2str(median(X(:,3),'omitnan'))])
disp(['Age, SD = ' num2str(std(X(:,3),'omitnan'))])
%% Headphones
disp(['Number of respondents with ''Are you using headphones now?'': Yes = ' num2str(sum(X(:,20)==2))])
disp(['Number of respondents with ''Are you using headphones now?'': No = ' num2str(sum(X(:,20)==1))])
disp(['Number of respondents with ''Are you using headphones now?'': I prefer not to respond = ' num2str(sum(isnan(X(:,20))))])
%% Problems with hearing
disp(['Number of respondents with ''Do you have problems with hearing?'': Yes = ' num2str(sum(X(:,21)==2))])
disp(['Number of respondents with ''Do you have problems with hearing?'': No = ' num2str(sum(X(:,21)==1))])
disp(['Number of respondents with ''Do you have problems with hearing?'': I prefer not to respond = ' num2str(sum(isnan(X(:,21))))])
%% Survey time
disp(['Survey time mean (minutes) after filtering = ' num2str(mean(X(:,18)/60))]);
disp(['Survey time median (minutes) after filtering = ' num2str(median(X(:,18)/60))]);
disp(['Survey time SD (minutes) after filtering = ' num2str(std(X(:,18)/60))]);
disp(['First survey start date after filtering = ' datestr(min(X(:,16)))]);
disp(['Last survey end date after filtering = ' datestr(max(X(:,17)))]);
end
